#!/usr/bin/python
# Copyright 2006, Marvell Semiconductor, Inc.
# Author: Luis Carlos Cobo <luisca@cozybit.com>
#
# This program is free software; you can redistribute it and/or
# modify it under the terms of the GNU Library General Public License
# as published by the Free Software Foundation; either version 2
# of the License, or (at your option) any later version.
#
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU Library General Public
# License along with this program; if not, write to the Free Software
# Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA
# 02111-1307, USA.

import socket, traceback, commands, syslog, os, sys

ANY_MPP_IP="172.31.255.254"
MPPREQ_PORT=16

host = ''
port = MPPREQ_PORT

mesh_iface_a='msh0'

# Default daemon parameters.
# File mode creation mask of the daemon.
UMASK = 0

# Default working directory for the daemon.
WORKDIR = "/"

# Default maximum for the number of available file descriptors.
MAXFD = 1024

# The standard I/O file descriptors are redirected to /dev/null by default.
if (hasattr(os, "devnull")):
    REDIRECT_TO = os.devnull
else:
    REDIRECT_TO = "/dev/null"

def createDaemon():
    """Detach a process from the controlling terminal and run it in the
    background as a daemon.
    """

    try:
        pid = os.fork()
    except OSError, e:
        raise Exception, "%s [%d]" % (e.strerror, e.errno)

    if (pid == 0):   # The first child.
        # We will almost certainly not receive a HUP at this point,
        # unless we stop executing due to failure.
        # import signal           # Set handlers for asynchronous events.
        # signal.signal(signal.SIGHUP, signal.SIG_IGN)
        
        try:
            pid = os.fork()# Fork a second child.
        except OSError, e:
            raise Exception, "%s [%d]" % (e.strerror, e.errno)
        
        if (pid == 0):   # The second child.
            # Since the current working directory may be a mounted filesystem,
            # we avoid the issue of not being able to unmount the filesystem at
            # shutdown time by changing it to the root directory.
            os.chdir(WORKDIR)
            os.umask(UMASK)
        else:
            os._exit(0)  # Exit parent (the first child) of the second child.
    else:
        os._exit(0)# Exit parent of the first child.
        
    # Close all open file descriptors.
    # Use the getrlimit method to retrieve the maximum file descriptor
    # number that can be opened by this process.  If there is not limit
    # on the resource, use the default value.
    #
    import resource     # Resource usage information.
    maxfd = resource.getrlimit(resource.RLIMIT_NOFILE)[1]
    if (maxfd == resource.RLIM_INFINITY):
        maxfd = MAXFD
        
    # Iterate through and close all file descriptors.
    for fd in range(0, maxfd):
        try:
            os.close(fd)
        except OSError: # ERROR, fd wasn't open to begin with (ignored)
            pass
        
    # Redirect the standard I/O file descriptors to the specified file.  Since
    # the daemon has no controlling terminal, most daemons redirect stdin,
    # stdout, and stderr to /dev/null.  This is done to prevent side-effects
    # from reads and writes to the standard I/O file descriptors.
    
    # This call to open is guaranteed to return the lowest file descriptor,
    # which will be 0 (stdin), since it was closed above.
    os.open(REDIRECT_TO, os.O_RDWR)    # standard input (0)
    
    # Duplicate standard input to standard output and standard error.
    os.dup2(0, 1)# standard output (1)
    os.dup2(0, 2)# standard error (2)
    return(0)

if __name__ == "__main__":
    retCode = createDaemon()

    syslog.openlog( 'mpp', 0, syslog.LOG_DAEMON )
    syslog.syslog( 'starting MPP server' )

    gateway=(commands.getoutput('ifconfig ' + mesh_iface_a + 
    	' | grep inet\ addr | awk \'{print $2}\'')).rsplit(':')[1]
    if not gateway or not len(gateway):
        syslog.syslog('Could not determine IP address of mesh interface.')
        os._exit(1)

    dnss=commands.getoutput('cat /etc/resolv.conf | awk \'{if ($1=="nameserver") print $2}\'').rsplit()
    if not dnss or not len(dnss):
        syslog.syslog('Could not determine DNS servers.')
        os._exit(1)

    mymessage_a='IPv4-0\n' + gateway + '\n' + dnss[0] + '\n'
    if len(dnss) > 1:
    	mymessage_a += dnss[1] + '\n'

    # Start MPP functionality in mesh firmware
    commands.getoutput('echo 1 > /sys/class/net/' + mesh_iface_a + '/libertas_mpp')
    
    # Alias ANY_MPP_IP to any mesh interface
    commands.getoutput('ifconfig ' + mesh_iface_a + ':0 ' + ANY_MPP_IP)

    syslog.syslog( 'starting to serve MPPREQ replies' )
    # Start serving MPPREQ replies
    s = socket.socket(socket.AF_INET, socket.SOCK_DGRAM)
    s.setsockopt(socket.SOL_SOCKET, socket.SO_REUSEADDR, 1)
    s.bind((host, port))

    while 1:
        try:
            message, address = s.recvfrom(8192)
            syslog.syslog( 'MPPREQUEST from:' + repr( address ) )
	    s.sendto(mymessage_a, address)

        except (KeyboardInterrupt, SystemExit):
            raise
        except:
            traceback.print_exc()
